[llvm-commits] CVS: llvm/include/Support/FileUtilities.h

Chris Lattner lattner at cs.uiuc.edu
Mon Dec 29 20:46:00 PST 2003


Changes in directory llvm/include/Support:

FileUtilities.h updated: 1.11 -> 1.12

---
Log message:

Further revisions of the FDHandle idea.  In this version we use ownership 
semantics that are the same as those used by std::auto_ptr.  This allows
copying of FDHandle's, but copying transfers ownership.


---
Diffs of the changes:  (+22 -5)

Index: llvm/include/Support/FileUtilities.h
diff -u llvm/include/Support/FileUtilities.h:1.11 llvm/include/Support/FileUtilities.h:1.12
--- llvm/include/Support/FileUtilities.h:1.11	Mon Dec 29 15:43:58 2003
+++ llvm/include/Support/FileUtilities.h	Mon Dec 29 20:45:16 2003
@@ -104,24 +104,41 @@
 
 
 /// FDHandle - Simple handle class to make sure a file descriptor gets closed
-/// when the object is destroyed.
+/// when the object is destroyed.  This handle acts similarly to an
+/// std::auto_ptr, in that the copy constructor and assignment operators
+/// transfer ownership of the handle.  This means that FDHandle's do not have
+/// value semantics.
 ///
 class FDHandle {
   int FD;
-  FDHandle(const FDHandle &);      // DO NOT IMPLEMENT
-  void operator=(const FDHandle&); // DO NOT IMPLEMENT
 public:
   FDHandle() : FD(-1) {}
   FDHandle(int fd) : FD(fd) {}
+  FDHandle(FDHandle &RHS) : FD(RHS.FD) {
+    RHS.FD = -1;       // Transfer ownership
+  }
+
   ~FDHandle() throw();
 
+  /// get - Get the current file descriptor, without releasing ownership of it.
+  int get() const { return FD; }
   operator int() const { return FD; }
 
   FDHandle &operator=(int fd) throw();
 
-  /// take - Take ownership of the file descriptor away from the FDHandle
+  FDHandle &operator=(FDHandle &RHS) {
+    int fd = RHS.FD;
+    RHS.FD = -1;       // Transfer ownership
+    return operator=(fd);
+  }
+
+  /// release - Take ownership of the file descriptor away from the FDHandle
   /// object, so that the file is not closed when the FDHandle is destroyed.
-  int take() throw();
+  int release() {
+    int Ret = FD;
+    FD = -1;
+    return Ret;
+  }
 };
 } // End llvm namespace
 





More information about the llvm-commits mailing list